from zohoSolCon import auth
import contacts
import contact_persons
import estimates


def test_contacts(token):
    # Contact creation
    data_object = {"contact_name": "Test Contact"}
    
    org_id="760546410"
    token, status, response = contacts.create_contact(token, org_id, data_object)
    print("Create Contact\n")
    print(response)
    contact_id = response['contact_id']
    token, page_context, records = contacts.get_contacts(token, org_id)
    print("Get Contacts\n")
    print(records)

    data_object = {'company_name': "Updated Company Name"}

    token, status, contact = contacts.update_contact(token, org_id, contact_id, data_object)
    print("Update Contact\n")
    print(contact)

    
    token, contact = contacts.get_contact(token, org_id, contact_id)
    print("Get Contact\n")
    print(contact)

    
    token, message = contacts.mark_status(token, org_id, contact_id, "inactive")

    print("Mark status\n")
    print(message)

    

def main():
    client_id = "1000.0VNUXZ0W47OW228LHBAJZ7RFLC6T1O"
    secret = "6920f0df429b5873910c4561dd9273b0344d1872cc"
    grant ="1000.1c95ac134c54c233dbdfff4c8dcbde90.da32af6be9a9b7e589e38458c5a0ce63"
    token = auth.Token(client_id, secret, grant_token=None, refresh_token='1000.7f41f25f0f8cc4c0086daed2f192ca48.d4381d05df5a5baf313447cdea55a3cb')
    print(token.refresh)
    test_contacts(token)



if __name__ == "__main__":
    main()

    



import requests 
import json
from datetime import datetime
import time
import csv
import os
from pathlib import Path


def make_header(token):
	return {
		"Authorization": f"Zoho-oauthtoken {token.access}",
		"Content-Type": "application/json"
	}


def create_bulk_read(token, module, **kwargs):
	url = "https://www.zohoapis.com/crm/bulk/v3/read"
	headers = make_header(token)
	data_object = {}
	callback = kwargs.get("callback")
	if callback is not None:
		data_object['callback'] = callback
	query_object = {}
	query_object['module'] = {'api_name': module}
	file_type = kwargs.get("file_type")
	if file_type is not None:
		file_type = "csv"
	query_object['file_type'] = file_type

	data_object['query'] = query_object
	#request_body = {"data": data_object}


	data = json.dumps(data_object).encode('utf-8')
	response = requests.post(url=url, headers=headers, data=data)
	print(response.status_code)
	print(response.content)
	if response.status_code == 401:
		token.generate()
		return create_bulk_read(token, module, **kwargs)

	else:
		content = json.loads(response.content.decode('utf-8'))
		data = content.get("data")[0]
		status = data.get("status")
		msg = data.get("message")
		details = data.get("details")
		return bulk_read_updates(token, details)




def fetch_bulk_read(token, job_obj):
	job_id = job_obj['id']
	url = f'https://www.zohoapis.com/crm/bulk/v3/read/{job_id}/result'
	headers = make_header(token)
	response = requests.get(url=url, headers=headers)
	if response.status_code == 401:
		token.generate()
		return fetch_bulk_read(token, job_obj)
	else:
		filename = Path(f"bulk_read{job_id}.zip")
		filename.write_bytes(response.content)
		input(f"Your data has been arrived: '{filename}' >>>>> ")
		return token, filename


def bulk_read_updates(token, job_details):

	status = job_details.get("status")
	job_id = job_details.get("id")
	url = f"https://www.zohoapis.com/crm/bulk/v3/read/{job_id}"
	headers = make_header(token)
	while status != "COMPLETED":
		os.system("clear")
		print(f"Your bulk read job currently has a status of {status}.")
		time.sleep(0.5)
		print("Fetching a status update in 300 seconds...")
		time.sleep(0.5)
		print("Press CTRL + C to skip the wait and fetch a status update >>> ")
		time_left = 300
		while time_left > 0:
			try:
				
				print(time_left)
				time.sleep(1)
				time_left -= 1

			except KeyboardInterrupt:
				
				os.system('clear')
				print("Fetching a status update")
				time.sleep(2.7)
				time_left = 0
		response = requests.get(url=url, headers=headers)
		if response.status_code == 401:
			token.generate()
			continue
		else:
			content =  json.loads(response.content.decode('utf-8'))
			data = content.get("data")[0]
			status = data.get("state")

	os.system("cear")
	print("Your bulk read job is marked as COMPLETED")	
	time.sleep(0.5)
	results = data.get("result")
	page = results['page']
	count = results['count']
	more = results['more_records']
	download_url = results['download_url']
	print(f"IMPORTANT download URL: {download_url}")
	job_obj = {
		"page": page, 
		"count": count,
		"more": more,
		"download_url": download_url,
		"status": status,
		"id": job_id


	}

	prompt = str(input("[y/Y] to begin downloading resource, [n/N] to just receive your bulk read URL >>> "))
	prompt = prompt.strip().upper()
	if "Y" in prompt:
		os.system('clear')
		print("Fetching your CSV Bulk Read job")
		token, response = fetch_bulk_read(token, job_obj)
		input("Enter to exit the script >>> ")

		

	else:
		os.system("clear")
		print(f"Download URL >>>> {download_url}")
		input("Enter to continue >>>> ")
		raise SystemExit()

	raise SystemExit()





			


import requests
import json


def make_header(token):
    return {
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }



def create_salesorder(token, org_id, customer_id, data_object):
    url = 'https://books.zoho.com/api/v3/salesorders'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object['customer_id'] = customer_id
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return create_salesorder(token, org_id, customer_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message"), content.get("salesorder")


def get_salesorders(token, org_id, **kwargs):
    url = 'htps://books.zoho.com/api/v3/salesorders'
    headers = make_header(token)
    params = kwargs
    params['organization_id'] = org_id

    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_salesorders(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("page_context"), content.get("salesorders")


def update_salesorder(token, org_id, salesorder_id, customer_id, data_object):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object['customer_id'] = customer_id
    data = json.dumps(data_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return update_salesorder(token, org_id, salesorder_id, customer_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('messsage'), content.get("salesorder")


def get_salesorder(token, org_id, salesorder_id, **kwargs):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}'
    headers = make_header(token)
    params = {
        'organization_id': org_id,
        'print': kwargs.get("print"),
        "accept": kwargs.get("accept")
    }
    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_salesorder(token, org_id, salesorder_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("salesorder")


def delete_salesorder(token, org_id, salesorder_id):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}'
    headers = make_header(token)
    params = {
        'organization_id': org_id,
    }
    response = requests.delete(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return delete_salesorder(token, org_id, salesorder_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message')

    

def mark_status(token, org_id, salesorder_id, status="open"):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/status/{status}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.post(url=url, headers=headers, params=params)
    if response.status_code == 401:
        token.generate()
        return mark_status(token, org_id, salesorder_id, status=status)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def email_salesorder(token, org_id, salesorder_id, to_mail_ids, **kwargs):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/email'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object = kwargs
    data_object['to_mail_ids'] = to_mail_ids


    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return email_salesorder(token, org_id, salesorder_id, to_mail_ids,**kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def get_salesorder_email_content(token, org_id, salesorder_id, **kwargs):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/email'
    headers = make_header(token)
    params = {
        'organization_id': org_id,
        'email_template_id': kwargs.get("email_template_id")
    }
    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_salesorder_email_content(token, org_id, salesorder_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")


def submit_for_approval(token, org_id, salesorder_id):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/submit'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.post(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return submit_for_approval(token, org_id, salesorder_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message')


def approve_salesorder(token, org_id, salesorder_id):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/approve'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.post(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return approve_salesorder(token, org_id, salesorder_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def update_billing_address(token, org_id, salesorder_id, address_object):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/address/billing'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data = json.dumps(address_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return update_billing_address(token, org_id, salesorder_id, address_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def update_shipping_address(token, org_id, salesorder_id, address_object):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/address/shipping'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data = json.dumps(address_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return update_shipping_address(token, org_id, salesorder_id, address_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def get_salesorder_templates(token, org_id):
    url = 'https://books.zoho.com/api/v3/salesorders/templates'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_salesorder_templates(token, org_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("templates")


def update_salesorder_template(token, org_id, salesorder_id, template_id):
    url = f'https://books.zoho.com/api/v3/salesorders/{salesorder_id}/templates/{template_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.put(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return update_salesorder_template(token, org_id, salesorder_id, template_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")




    



import requests
import json


def make_header(token):
    return {
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }


def create_contact(token, org_id, data_object):
    url = f'https://inventory.zoho.com/api/v1/contacts?organization_id={org_id}'
    headers = make_header(token)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return create_contact(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message"), content.get("contact")


def get_contacts(token, org_id, **kwargs):
    url = f'https://inventory.zoho.com/api/v1/contacts?organization_id={org_id}'
    headers = make_header(token)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_contacts(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('contacts')


    
def update_contact(token, org_id, contact_id, data_object):
    url = f'https://inventory.zoho.com/api/v1/contacts/{contact_id}?organization_id={org_id}'
    headers = make_header(token)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return update_contact(token, org_id, contact_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message"), content.get("contact")


def get_contact(token, org_id, contact_id):
    url = f'https://inventory.zoho.com/api/v1/contacts/{contact_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.get(url=url, headers=headers, params=params)
    if response.status_code == 401:
        token.generate()
        return get_contact(token, org_id, contact_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('contact')


def delete_contact(token, org_id, contact_id):
    url = f'https://inventory.zoho.com/api/v1/contacts/{contact_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.delete(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return delete_contact(token, org_id, contact_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def toggle_status(token, org_id, contact_id, status="active"):
    url = f'https://inventory.zoho.com/api/v1/contacts/{contact_id}/{status}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.post(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return toggle_status(token, org_id, contact_id, status=status)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message')


def email_statement(token, org_id, contact_id, subject, body, **kwargs):
    url = f'https://inventory.zoho.com/api/v1/contacts/{contact_id}/statements/email'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object = {
        'send_from_org_email_id': kwargs.get('send_from_org_email_id'),
        'to_mail_ids': kwargs.get('to_mail_ids'),
        'cc_mail_ids': kwargs.get("cc_mail_ids"),
        "subject": subject,
        "body": body
    }
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return email_statement(token, org_id, contact_id, subject, body, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")



    



import auth
import records
import related_list



client_id = "1000.0VNUXZ0W47OW228LHBAJZ7RFLC6T1O"
client_secret = "6920f0df429b5873910c4561dd9273b0344d1872cc"
grant = "1000.c7931eabcf1c014777d693f939fb7521.e3dc2c451c4a977ec3e0e6cb231b0178"

token = auth.Token(client_id=client_id, client_secret=client_secret, grant_token=grant)


token, response = records.get_records(token, "Accounts", page="1", per_page="1")

account_id = response[0]['id']

token, response = records.get_records(token, "Products", page="1", per_page="1")

prod_id = response[0]['id']

token, status, response = related_list.update_related_record(token, "Accounts", account_id, "Products", prod_id, {})

print(response)



import requests
import json


def make_header(token, org_id):
    return {
        'orgId': org_id,
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }


def get_profiles(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/profiles'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_profiles(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))

        return token, content.get("data")


def get_profile_count(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/profiles/count'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_profile_count(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("count")


def get_profile(token, org_id, profile_id):
    url = f'https://desk.zoho.com/api/v1/profiles/{profile_id}'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_profile(token, org_id, profile_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def clone_profile(token, org_id, profile_id, profile_name, **kwargs):
    url = f'https://desk.zoho.com/api/v1/profiles/{profile_id}/clone'
    headers = make_header(token, org_id)

    data_object = {'name': profile_name}
    data_object['description'] = kwargs.get('description')

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return clone_profile(token, org_id, profile_id, profile_name, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def update_profile(token, org_id, profile_id, permissions_object, **kwargs):
    url = f'https://desk.zoho.com/api/v1/profiles/{profile_id}'
    headers = make_header(token, org_id)

    data_object = {"permissions": permissions_object}
    data_object['description'] = kwargs.get('description')

    data = json.dumps(data_object).encode('utf-8')

    response = requests.patch(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return update_profile(token, org_id, profile_id, permissions_object, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def delete_profile(token, org_id, profile_id, transfer_to_id):
    url = f'https://desk.zoho.com/api/v1/profiles/{profile_id}/delete'
    headers = make_header(token, org_id)

    data_object = {'transferToProfileId': transfer_to_id}
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return delete_profile(token, org_id, profile_id, transfer_to_id)

    else:
        return token, response.status_code


def get_agents_by_profile(token, org_id, profile_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/profiles/{profile_id}/agents'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_agents_by_profile(token, org_id, profile_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("agents")





import requests
import json


def make_header(token):
    return {
        'Authorization': 'Zoho-oauthtoken {token.access}'
    }


def create_organization(token, org_id, data_object):
    url = f'https://inventory.zoho.com/api/v1/organizations?organization_id={org_id}'
    headers = make_header(token)
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return create_organization(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message"), content.get("organization")


def get_organizations(token, org_id):
    url = f'https://inventory.zoho.com/api/v1/organizations?organization_id={org_id}'
    headers = make_header(token)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_organizations(token, org_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("organizations")





import requests
import json


def make_header(token):
    return {
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }



def create_composite_item(token, org_id, name, mapped_items, **kwargs):
    url = 'https://inventory.zoho.com/api/v1/compositeitems'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object = kwargs

    data_object['mapped_items'] = mapped_items
    data_object['name'] = name

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return create_composite_item(token, org_id, name, mapped_items, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message'), content.get("composite_item")


def get_composite_items(token, org_id, **kwargs):
    url = 'https://inventory.zoho.com/api/v1/compositeitems'
    headers = make_header(token)
    params = kwargs
    params['organization_id'] = org_id

    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_composite_items(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("composite_items")



def update_composite_item(token, org_id, item_id, data_object):
    url = f'https://inventory.zoho.com/api/v1/compositeitems/{item_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}
    data = json.dumps(data_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return update_composite_item(token, org_id, item_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message'), content.get("composite_item")


def get_composite_item(token, org_id, item_id):
    url = f'https://inventory.zoho.com/api/v1/compositeitems/{item_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}
    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_composite_item(token, org_id, item_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("composite_item")


def delete_composite_item(token, org_id, item_id):
    url = f'https://inventory.zoho.com/api/v1/compositeitems/{item_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}
    response = requests.delete(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return delete_composite_item(token, org_id, item_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def toggle_status(token, org_id, item_id, status="active"):
    url = f'https://inventory.zoho.com/api/v1/compositeitems/{item_id}/{status.lower()}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.post(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return toggle_status(token, org_id, item_id, status=status)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def create_bundle(token, org_id, reference_number, date, composite_item_id, quantity, line_items, is_completed=True, **kwargs):
    url = 'https://inventory.zoho.com/api/v1/bundles'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object = {
        'reference_number': reference_number,
        'date': date,
        'composite_item_id': composite_item_id,
        'quantity_to_bundle': quantity,
        'line_items': line_items,
        'is_completed': is_completed,
        'description': kwargs.get('description'),
        'composite_item_name': kwargs.get("composite_item_name"),
        'composite_item_sku': kwargs.get("composite_item_sku")
    }
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)
    if response.status_code == 401:
        token.generate()
        return create_bundle(token, org_id, reference_number, date, composite_item_id, quantity, line_items, is_completed=is_completed, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message"), content.get("bundle_id")


def bundle_history(token, org_id, composite_item_id):
    url = 'https://inventory.zoho.com/api/v1/bundles'
    headers = make_header(token)
    params = {
        'organization_id': org_id,
        'composite_item_id': composite_item_id
    }
    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return bundle_history(token, org_id, composite_item_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("bundles")



def get_bundle(token, org_id, bundle_id):
    url = f'https://inventory.zoho.com/api/v1/bundles/{bundle_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}
    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_bundle(token, org_id, bundle_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('bundle')


def delete_bundle(token, org_id, bundle_id):
    url = f'https://inventory.zoho.com/api/v1/bundles/{bundle_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}
    response = requests.delete(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return delete_bundle(token, org_id, bundle_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('messsage')
    



import requests
import json
import os


def make_header(token):
    return {
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }


def get_attachments(token, module, record_id, **kwargs):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/Attachments'
    headers = make_header(token)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_attachments(token, module, record_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")



def upload_attachment(token, module, record_id, file_name):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/Attachments'
    headers = make_header(token)

    with open(file_name, 'rb') as f:
        request_body = {'file': f}
        response = requests.post(url=url, files=request_body, headers=headers)

    if response.status_code == 401:
        token.generate()
        return upload_attachment(token, module, record_id, file_name)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content.get("data")



def download_attachment(token, module, record_id, attachment_id, destination_dir):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/Attachments/{attachment_id}'
    headers = make_header(token)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return download_attachment(token, module, record_id, attachment_id, destination_dir)

    else:
        if "Content-Type" in response.headers:
            content_type = response.headers['Content-Type']

            if content_type == 'application/json':
                return token, response.json

            else:
                if "Content-Disposition" in response.headers:
                    file_name = ''
                    content_disposition = response.headers["Content-Disposition"]

                    if "'" in content_disposition:
                        start_index = content_disposition.rindex("'")
                        file_name = content_disposition[start_index + 1:]

                    elif '"' in content_disposition:
                        start_index = content_disposition.rindex('=')
                        file_name = content_disposition[start_index + 1:].replace('"', '')

                    destination_file = os.path.join(destination_dir, file_name)

                    with open(destination_file, 'wb') as f:
                        for chunk in response:
                            f.write(chunk)

                    return token, destination_file


def delete_attachment(token, module, record_id, attachment_id):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/Attachments/{attachment_id}'
    headers = make_header(token)

    response = requests.delete(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return delete_attachment(token, module, record_id, attachment_id)


    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")



def delete_photo(token, module, record_id):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/photo'
    headers = make_header(token)

    response = requests.delete(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return delete_photo(token, module, record_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")


def upload_photo(token, module, record_id, file_name):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/photo'
    headers = make_header(token)

    with open(file_name, 'rb') as f:
        request_body = {'file': f}
        response = requests.post(url=url, files=request_body, headers=headers)

    if response.status_code == 401:
        token.generate()
        return upload_photo(token, module, record_id, file_name)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content.get("data")


def download_photo(token, module, record_id, destination_dir):
    url = f'https://www.zohoapis.com/crm/v2.1/{module}/{record_id}/photo'
    headers = make_header(token)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return download_photo(token, module, record_id, destination_dir)

    else:
        if "Content-Type" in response.headers:
            content_type = response.headers['Content-Type']

            if content_type == 'application/json':
                return token, response.json

            else:
                if "Content-Disposition" in response.headers:
                    file_name = ''
                    content_disposition = response.headers["Content-Disposition"]

                    if "'" in content_disposition:
                        start_index = content_disposition.rindex("'")
                        file_name = content_disposition[start_index + 1:]

                    elif '"' in content_disposition:
                        start_index = content_disposition.rindex('=')
                        file_name = content_disposition[start_index + 1:].replace('"', '')

                    destination_file = os.path.join(destination_dir, file_name)

                    with open(destination_file, 'wb') as f:
                        for chunk in response:
                            f.write(chunk)

                    return token, destination_file


    
    



import requests
import json


def make_header(token):
    return {
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }


def create_item(token, org_id, data_object):
    url = 'https://inventory.zoho.com/api/v1/items'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return create_item(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message"), content.get("item")


def get_items(token, org_id):
    url = 'https://inventory.zoho.com/api/v1/items'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_items(token, org_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("items")


def update_item(token, org_id, item_id, data_object):
    url = f'https://inventory.zoho.com/api/v1/items/{item_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data = json.dumps(data_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return update_item(token, org_id, item_id, data_object)

    else:
        content = json.loads(repsonse.content.decode('utf-8'))
        return token, content.get('message'), content.get("item")


def get_item(token, org_id, item_id):
    url = f'https://inventory.zoho.com/api/v1/items/{item_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_item(token, org_id, item_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('item')


def delete_item(token, org_id, item_id):
    url = f'https://inventory.zoho.com/api/v1/items/{item_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.delete(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return delete_item(token, org_id, item_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message')


    

    



import requests
import json


def make_header(token, org_id):
    return {
        "orgId": org_id,
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }


def get_agent(token, org_id, agent_id):
    url = f'https://desk.zoho.com/api/v1/agents/{agent_id}'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_agent(token, org_id, agent_id)

    else:
        data = json.loads(response.content.decode('utf-8'))
        return token, data


def get_agents(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/agents'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_agents(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def get_agent_count(token, org_id, **kwargs):
    url = 'https://deks.zoho.com/api/v1/agents/count'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_agent_count(token, org_id, **kwargs)

    else:
        content = json.loads(respnse.content.decode('utf-8'))
        return token, content.get("count")


def activate_agents(token, org_id, agent_id_list):
    url = 'https://desk.zoho.com/api/v1/agents/activate'
    headers = make_header(token, org_id)

    data_object = {'agentIds': agent_id_list}
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return activate_agents(token, org_id, agent_id_list)

    else:
        return token, response


def deactivate_agent(token, org_id, agent_id):
    url = f'https://desk.zoho.com/api/v1/agents/{agent_id}/deactivate'
    headers = make_header(token, org_id)

    response = requests.post(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return deactivate_agent(token, org_id, agent_id)

    else:
        return token, response


def add_agent(token, org_id, data_object):
    url = 'https://desk.zoho.com/api/v1/agents'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return add_agent(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def update_agent(token, org_id, agent_id, data_object):
    url = f'https://desk.zoho.com/api/v1/agents/{agent_id}'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return update_agent(token, org_id, agent_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def delete_unconfirmed_agents(token, org_id, agent_id_list):
    url = 'https://desk.zoho.com/api/v1/agents/deleteUnconfirmed'
    headers = make_header(token, org_id)

    data_object = {'agentIds': agent_id_list}
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return delete_unconfirmed_agents(token, org_id, agent_id_list)
    else:
        return token, response.status_code


def delete_agent(token, org_id, agent_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/agents/{agent_id}/delete'
    headers = make_header(token, org_id)

    data = json.dumps(kwargs).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return delete_agent(token, org_id, agent_id, **kwargs)

    else:
        return token, response.status_code


def get_agent_by_email(token, org_id, email_id):
    url = f'https://desk.zoho.com/api/v1/agents/email/{email_id}'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_agent_by_email(token, org_id, email_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


    



import requests
import json


def format_header(token):
	return {
		"Authorization": f"Zoho-oauthtoken {token.access}"
	}
	
	
def create_estimate(token, org_id, data_object, line_items):
	url = f'https://books.zoho.com/api/v3/estimates?organization_id={org_id}'
	headers = format_header(token)
	
	data_object['line_items'] = line_items
	
	data = json.dumps(data_object).encode('utf-8')
	
	response = requests.post(url=url, headers=headers, data=data)
	
	if response.status_code == 400:
		token.generate()
                return create_estimate(token, org_id, data_object, line_items)
		
	else:
		content = json.loads(response.content.decode('utf-8'))
		try:
			estimate = content['estimate']
		except Exception as e:
			estimate = {"status":  response.status_code, "error": str(e)}
		return token, content.get("message"), estimate
		
		

def get_estimates(token, org_id, **kwargs):
	url = f'https://books.zoho.com/api/v3/estimates?organization_id={org_id}'
	headers = format_header(token)
	
	response = requests.get(url=url, headers=headers, params=kwargs)
	
	if response.status_code == 400:
		token.generate()
		return get_estimates(token, org_id, **kwargs)
		
	else:
		content = json.loads(response.content.decode('utf-8'))
		try:
			estimates = content['estimates']
			page_context = content['page_context']
		except Exception as e:
			estimates = [{"status": response.status_code, "error": str(e)}]
		
		return token, page_context, estimates
		
	
def update_estimate(token, org_id, estimate_id, data_object):
	url = f'https://books.zoho.com/api/v3/estimates/{estimate_id}?organization_id={org_id}'
	headers = {
		'Authorization': f'Zoho-oauthtoken {token.access}'
	}
	
	data = json.dumps(data_object).encode("utf-8")
	
	response = requests.put(url=url, headers=headers, data=data)
	
	if response.status_code == 400:
		token.generate()
		return update_estimate(token, org_id, estimate_id, data_object)
		
	else:
		content = json.loads(response.content.decode('utf-8'))
		try:
			estimate = content['estimate']
		except Exception as e:
			estimate = {"status": response.status_code, "error": str(e)}
		
		return token, content.get("message"), estimate
		
		

def get_estimate(token, org_id, estimate_id, **kwargs):
	url = f'https://books.zoho.com/api/v3/estimates/{estimate_id}?organization_id={org_id}'
	headers = {
		'Authorization': f'Zoho-oauthtoken {token.access}'
	}
	
	response = requests.get(url=url, headers=headers, params=kwargs)
	
	if response.status_code == 400:
		token.generate()
		return get_estimate(token, org_id, estimate_id, **kwargs)
	
	else:
		content = json.loads(response.content.decode('utf-8'))
		try:
			estimate = content['estimate']
		except Exception as e:
			estimate = {'status': response.status_code, 'error': str(e)}
			
		return token, estimate
		
		
def delete_estimate(token, org_id, estimate_id):
	url = f'https://books.zoho.com/api/v3/estimates/{estimate_id}?organization_id={org_id}'
	headers = {
		'Authorization': f'Zoho-oauthtoken {token.access}'
	}
	
	response = requests.delete(url=url, headers=headers)
	
	if response.status_code == 400:
		token.generate()
		return delete_estimate(token, org_id, estimate_id)
		
	else:
		content = json.loads(response.content.decode('utf-8'))
		return token, content.get("message")
		

def mark_status(token, org_id, estimate_id, status):
	url = f'https://books.zoho.com/api/v3/estimates/{estimate_id}/status/{status}?organization_id={org_id}'
	headers = {
		'Authorization': f'Zoho-oauthtoken {token.access}'
	}
	
	response = requests.post(url=url, headers=headers)
	
	if response.status_code == 400:
		token.generate()
		return mark_status(token, org_id, estimate_id, status)
		
	else:
		content = json.loads(response.content.decode('utf-8'))
		return token, content.get("message")
		

def estimate_approval(token, org_id, estimate_id, action):
	url = f'https://books.zoho.com/api/v3/estimates/{estimate_id}/{action}?organization_id={org_id}'
	headers = {
		'Authorization': f'Zoho-oauthtoken {token.access}'
	}
	
	response = requests.post(url=url, headers=headers)
	
	if response.status_code == 400:
		token.generate()
		return submit_for_approval(token, org_id, estimate_id)
		
	else:
		content = json.loads(response.content.decode('utf-8'))
		return token, content.get("message")
		

def email_estimate(token, org_id, estimate_id, to_mail_ids, **kwargs):
        url = f'https://books.zoho.com/api/v3/estimates/{estimate_id}/email'
        headers = format_header(token)
        params = {'organization_id': org_id}

        data_object = kwargs
        data_object['to_mail_ids'] = to_mail_ids

        data = json.dumps(data_object).encode('utf-8')
        response = requests.post(url=url, headers=headers, params=params, data=data)
        if response.status_code == 401:
                token.generate()
                return email_estimate(token org_id, estimate_id, to_mail_ids, **kwargs)
        else:
                content = json.loads(response.content.decode('utf-8'))
                return token, content.get('message')
                
                
        

def mass_action(token, org_id, callback, **kwargs):
        next_page = True
        page = 1
        iterated = 0
        parameters = kwargs
        while next_page:
                url = f'https://books.zoho.com/api/v3/estimates?organization_id={org_id}'
                headers = {
                        'Authorization': f'Zoho-oauthtoken {token.access}'
                }
                parameters['page'] = str(page)
                parameters['per_page'] = '200'

                response = requests.get(url=url, headers=headers, params=parameters)
                if response.status_code == 400:
                        token.generate()
                        continue

                content = json.loads(response.content.decode('utf-8'))
                estimates = content['estimates']

                for record in estimates:
                        token, callback_response = callback(token, org_id, record)
                        print(callback_response)
                        iterated += 1
                        print(f'{iterated} Records Iterated')

                page_context = content['page_context']
                page += 1
                next_page = page_context['has_more_page']
        return token, f"{iterated} Records iterated \n===========\n Mass Operations complete."
        
                        
                



import requests
import json


def make_header(token, org_id):
    return {
        'orgId': org_id,
        'Authorization': 'Zoho-oauthtoken {token.access}'
    }


def get_roles(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/roles'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_roles(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")

    
def get_agents_by_role(token, org_id, role_id):
    url = f'https://desk.zoho.com/api/v1/roles/{role_id}/agents'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_agents_by_role(token, org_id, role_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("associatedAgents")



def create_role(token, org_id, role_name, share_with_peers=False, **kwargs):
    url = 'https://desk.zoho.com/api/v1/roles'
    headers = make_header(token, org_id)

    data_object = {
        'name': role_name,
        'shareDataWithPeers': share_with_peers
    }

    data_object['description'] = kwargs.get("description")
    data_object['reportsTo'] = kwargs.get("reportsTo")

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return create_role(token, org_id, role_name, share_with_peers=share_with_peers, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def update_role(token, org_id, role_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/roles/{role_id}'
    headers = make_header(token, org_id)

    data_object = kwargs
    data = json.dumps(data_object).encode('utf-8')

    response = requests.patch(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return update_role(token, org_id, role_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content



def delete_role(token, org_id, role_id, transfer_to_id):
    url = f'https://desk.zoho.com/api/v1/roles/{role_id}/delete'
    headers = make_header(token, org_id)

    data_object = {'transferToRoleId': transfer_to_id}
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return delete_role(token, org_id, role_id, transfer_to_id)

    else:
        return token, response.status_code


def get_role(token, org_id, role_id):
    url = f'https://desk.zoho.com/api/v1/roles/{role_id}'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_role(token, org_id, role_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def get_role_count(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/roles/count'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_role_count(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("count")





import requests
import json


def make_header(token):
    return {
        'Authorization': f'Zoho-oauthtoken {token.access}'
    }


def create_invoice(token, org_id, customer_id, data_object):
    url = 'https://books.zoho.com/api/v3/invoices'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object['customer_id'] = customer_id
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return create_invoice(token, org_id, customer_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message'),content.get("invoice")
    

def get_invoices(token, org_id, **kwargs):
    url = 'https://books.zoho.com/api/v3/invoices'
    headers = make_header(token)
    params = kwargs
    params['organization_id'] = org_id

    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_invoices(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("page_context"), content.get("invoices")



def update_invoice(token, org_id, invoice_id, customer_id, data_object):
    url = f'https://books.zoho.com/apiv/3/invoices/{invoice_id}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object['customer_id'] = customer_id
    data = json.dumps(data_object).encode('utf-8')

    response = requests.put(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return update_invoice(token, org_id, invoice_id, customer_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message"), content.get("invoice")


def get_invoice(token, org_id, invoice_id, **kwargs):
    url = f'https://books.zoho.com/api/v3/invoices/{invoice_id}'
    headers = make_header(token)
    params = {
        'organization_id': org_id,
        'print': kwargs.get("print"),
        "accept": kwargs.get("accept")
    }
    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return get_invoice(token, org_id, invoice_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('invoice')

def delete_invoice(token, org_id, invoice_id):
    url = f'https://books.zoho.com/api/v3/invoices/{invoice_id}'
    headers = make_header(token)
    params = {
        'organization_id': org_id
    }
    response = requests.delete(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return delete_invoice(token, org_id, invoice_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('message')



def mark_status(token, org_id, invoice_id, status):
    url = f'https://books.zoho.com/api/v3/invoices/{invoice_id}/status/{status}'
    headers = make_header(token)
    params = {'organization_id': org_id}

    response = requests.post(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return mark_status(token, org_id, invoice_id, status)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")


def email_invoice(token, org_id, invoice_id, to_mail_ids, **kwargs):
    url = f'https://books.zoho.com/api/v3/invoices/{invoice_id}/email'
    headers = make_header(token)
    params = {'organization_id': org_id}

    data_object = kwargs
    data_object['to_mail_ids'] = to_mail_ids
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, params=params, data=data)

    if response.status_code == 401:
        token.generate()
        return email_invoice(token, org_id, invoice_id, to_mail_ids, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("message")

    
    






import requests
import json


def make_header(token, org_id):
    return {
        'orgId': org_id,
        'Authorization': 'Zoho-oauthtoken {token.access}'
    }


def get_department(token, org_id, department_id):
    url = f'https://desk.zoho.com/api/v1/departments/{department_id}'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_department(token, org_id, department_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def get_departments(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/departments'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_departments(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")


def get_agents_by_department(token, org_id, department_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/departments/{department_id}/agents'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return get_agents_by_department(token, org_id, departmend_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def get_role_count(token, org_id, is_enabled=True):
    url = 'https:///desk.zoho.com/api/v1/departments/count'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params={"isEnabled":is_enabled})

    if response.status_code == 401:
        token.generate()
        return get_role_count(token, org_id, is_enabled=is_enabled)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('count')


def create_department(token, org_id, data_object):
    url = 'https://desk.zoho.com/api/v1/departments'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return create_department(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def update_department(token, org_id, department_id, data_object):
    url = f'https://desk.zoho.com/api/v1/departments/{department_id}'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.patch(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return update_department(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def disable_department(token, org_id, department_id, transfer_to_id, email_to_id=None):
    url = f'https://desk.zoho.com/api/v1/departments/{department_id}/disable'
    headers = make_header(token, org_id)

    data_object = {
        'emailNewDepartment': email_to_id,
        'agentNewDepartment': transfer_to_id
    }

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return disable_department(token, org_id, department_id, transfer_to_di, email_to_id=email_to_id)

    else:
        return token, response.status_code


def enable_department(token, org_id, department_id):
    url = f'https://desk.zoho.com/api/v1/departments/{department_id}/enable'
    headers = make_header(token, org_id)

    response = requests.post(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return enable_department(token, org_id, departmend_id)

    else:
        return token, response.status_code


def associate_to_department(token, org_id, department_id, agent_id_list):
    url = f'https://desk.zoho.com/api/v1/departments/{department_id}/associateAgents'
    headers = make_header(token, org_id)

    data_object = {'agentIds': agent_id_list}

    data = json.dumps(data_object).encode('utf-8')

    




import requests
import json


def make_header(token, org_id):
    return {
        'orgId': org_id,
        'Authorization': 'Zoho-oauthtoken {token.access}'
    }



def create_team(token, org_id, data_object):
    url = 'https://desk.zoho.com/api/v1/teams'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return create_team(token, org_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content



def update_team(token, org_id, team_id, data_object):
    url = f'https://desk.zoho.com/api/v1/teams/{team_id}'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode('utf-8')

    response = requests.patch(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return update_team(token, org_id, team_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def delete_team(token, org_id, team_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/teams/{team_id}/deleteTeam'
    headers = make_header(toke, org_id)
    data_obj = kwargs
    data = json.dumps(data_obj).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)
    if response.status_code == 401:
        token.generate()
        return delete_team(token, org_id, team_id, **kwargs)

    else:
        return token, response.status_code


def get_team(token, org_id, team_id):
    url = f'https://desk.zoho.com/api/v1/teams/{team_id}'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_team(token, org_id, team_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def get_teams(token, org_id):
    url = 'https://desk.zoho.com/api/v1/teams'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_teams(token, org_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("teams")


def get_team_members(token, org_id, team_id):
    url = f'https://desk.zoho.com/api/v1/teams/{team_id}/members'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_team_members(token, org_id, team_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("members")


def get_associable_teams(token, org_id, team_id):
    url = f'https://desk.zoho.com/api/v1/teams/{team_id}/associables'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_associable_teams(token, org_id, team_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('teams')


def get_teams_by_agent(token, org_id, agent_id):
    url = f'https://desk.zoho.com/api/v1/agents/{agent_id}/teams'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_teams_by_agent(token, org_id, agent_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('teams')


def get_teams_by_role(token, org_id, role_id):
    url = f'https://desk.zoho.com/api/v1/roles/{role_id}/teams'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_teams_by_role(token, org_id, role_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('teams')


def get_teams_by_department(token, org_id, department_id):
    url = f'https://desk.zoho.com/api/v1/departments/{department_id}/teams'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_teams_by_department(token, org_id, department_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("teams")






import requests
import json


def make_header(token):
    return {
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }


def get_org_details(token):
    url = "https://www.zohoapis.com/crm/v2.1/org"

    headers = make_header(token)
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_org_details(token)

    else:
        content = json.loads(response.content.decode('utf-8'))
        org = content.get("org")
        return token, org


def upload_org_photo(token, file_name):
    url = 'https://www.zohoapis.com/crm/v2.1/org/photo'
    headers = make_header(token)

    with open(file_name, 'rb') as file:
        request_body = {'file': file}
        response = requests.post(url=url, files=request_body, headers=headers)

    if response.status_code == 401:
        token.generate()
        return upload_org_photo(token, file_name)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content



import requests
import json
from datetime import datetime


def make_header(token, org_id):
    return {
        "orgId": org_id,
        "Authorization": f"Zoho-oauthtoken {token.access}"
    }


def get_ticket(token, org_id, ticket_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/tickets/{}'.format(ticket_id)
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_ticket(token, org_id, ticket_id, **kwargs)

    else:
        data = json.loads(response.content.decode('utf-8'))
        return token, data


def get_tickets(token, org_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/tickets'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return get_tickets(token, org_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def create_ticket(token, org_id, data_object):
    url = 'https://desk.zoho.com/api/v1/tickets'
    headers = make_header(token, org_id)

    data = json.dumps(data_object).encode("utf-8")
    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return create_ticket(token, org_id, data_object)
    

def update_ticket(token, org_id, ticket_id, data_object):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}'
    data = json.dumps(data_object).encode('utf-8')
    headers = make_header(token, org_id)
    response = requests.patch(url=url, headers=headers, data=data)
    if response.status_code == 401:
        print("Auth")
        token.generate()
        return update_ticket(token, org_id, ticket_id, data_object)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def trash_ticket(token, org_id, ticket_id_list):
    url = 'https://desk.zoho.com/api/v1/tickets/moveToTrash'
    headers = make_header(token, org_id)
    
    data_object = {'ticketIds': ticket_id_list}

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return trash_ticket(token, org_id, ticket_id_list)

    else:
        return token, response.status_code


def move_ticket(token, org_id, ticket_id, department_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/move'
    headers = make_header(token, org_id)
    data_object = {'departmentId': department_id}

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return move_ticket(token, org_id, ticket_id, department_id)

    else:
        return token, response.status_code


def mark_read(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/markAsRead'
    headers = make_header(token, org_id)
    response = requests.post(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return mark_read(token, org_id, ticket_id)

    else:
        return token, response.status_code


def mark_unread(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/markAsUnRead'
    headers = make_header(token, org_id)
    response = requests.post(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return mark_unread(token, org_id, ticket_id)

    else:
        return token, response.status_code


def ticket_history(token, org_id, ticket_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/History'
    headers = make_header(token, org_id)
    
    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return ticket_history(token, org_id, ticket_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content.get('data')


def ticket_resolution(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/resolution'
    headers = make_header(token, org_id)
    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        print("Auth")
        token.generate()
        return ticket_resolution(token, org_id, ticket_id)

    
def merge_tickets(token, org_id, ticket_id, merge_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/merge'
    headers = make_header(token, org_id)

    data_object = {
        'ids': [merge_id],
        'source': kwargs.get('source')
    }
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return merge_tickets(token, org_id, ticket_id, merge_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content

    
def split_ticket(token, org_id, ticket_id, thread_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/threads/{thread_id}/split'
    headers = make_header(token, org_id)

    response = requests.post(url=url, headers=headers)
    if response.status_code == 401:
        token.generate()
        return split_ticket(token, org_id, ticket_id, thread_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def bulk_update_tickets(token, org_id, ticket_id_list, field_name, **kwargs):
    url = 'https://desk.zoho.com/api/v1/tickets/updateMany'
    headers = make_header(token, org_id)

    data_object = {
        'fieldName': field_name,
        'ids': ticket_id_list,
        'fieldValue': kwargs.get("fieldValue"),
        'isCustomField': kwargs.get("isCustomField")
    }

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return bulk_update_tickets(token, org_id, ticket_id_list, field_name, **kwargs)

    else:
        return token, response.status_code



def close_tickets(token, org_id, ticket_id_list):
    url = 'https://desk.zoho.com/api/v1/tickets/closeTickets'
    headers = make_header(token, org_id)

    data_object = {'ids': ticket_id_list}
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return close_tickets(token, org_id, ticket_id_list)

    else:
        return token, response.status_code


def mark_as_spam(token, org_id, ticket_id_list, **kwargs):
    url = 'https://desk.zoho.com/api/v1/tickets/markSpam'
    headers = make_header(token, org_id)

    data_object = kwargs
    data_object['ids'] = ticket_id_list

    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return mark_as_spam(token, org_id, ticket_id_list, **kwargs)

    else:
        return token, response.status_code


def count_in_view(token, org_id, view_id, department_id, **kwargs):
    url = 'https://desk.zoho.com/api/v1/ticketQueueView/count'
    headers = make_header(token, org_id)

    params = {
        'viewId': view_id,
        'departmentId': department_id,
        'agentId': kwargs.get("agentId")
    }
    response = requests.get(url=url, headers=headers, params=params)

    if response.status_code == 401:
        token.generate()
        return count_in_view(token, org_id, view_id, department_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('count')


def agents_ticket_count(token, org_id, agent_id_list, **kwargs):
    url = 'https://desk.zoho.com/api/v1/agentsTicketsCount?agentIds='
    for agent_id in agent_id_list:
        url += f"{agent_id},"

    url[-1] = ""
    headers = make_header(token, org_id)

    response = requests.get(url=url, token=token, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return agents_ticket_count(token, org_id, agent_id_list, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get("data")


def ticket_history(token, org_id, ticket_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/History'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return ticket_history(token, org_id, ticket_id, **kwargs)

    else:
        content = json.loads(response.content.decode("utf-8"))
        return token, content.get("data")


def ticket_resolution(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/resolution'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return ticket_resolution(token, org_id, ticket_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def resolution_history(token, org_id, ticket_id, **kwargs):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/resolutionHistory'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers, params=kwargs)

    if response.status_code == 401:
        token.generate()
        return resolution_history(token, org_id, ticket_id, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content.get('data')


def update_resolution(token, org_id, content, **kwargs):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/resolution'
    headers = make_header(token, org_id)

    data_object = {
        'content': content,
        'isNotifyContact': kwargs.get("isNotifyContact")
    }
    data = json.dumps(data_object).encode('utf-8')

    response = requests.patch(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return update_resolution(token, org_id, content, **kwargs)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, response.status_code, content


def delete_resolution(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/resolution'
    headers = make_header(token, org_id)

    response = requests.delete(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return delete_resolution(token, org_id, ticket_id)

    else:
        return token, response.status_code


def get_metrics(token, org_id, ticket_id):
    url = f'https://desk.zoho.com/api/v1/tickets/{ticket_id}/metrics'
    headers = make_header(token, org_id)

    response = requests.get(url=url, headers=headers)

    if response.status_code == 401:
        token.generate()
        return get_metrics(token, org_id, ticket_id)

    else:
        content = json.loads(response.content.decode('utf-8'))
        return token, content


def delete_spam_tickets(token, org_id, department_id):
    url = 'https://desk.zoho.com/api/v1/tickets/emptySpam'
    headers = make_header(token, org_id)

    data_object = {'departmentId': department_id}
    data = json.dumps(data_object).encode('utf-8')

    response = requests.post(url=url, headers=headers, data=data)

    if response.status_code == 401:
        token.generate()
        return delete_spam_tickets(token, org_id, department_id)

    else:
        return token, response.status_code



def mass_action(token, org_id, callback, **kwargs):
    url = 'https://desk.zoho.com/api/v1/tickets'
    headers = make_header(token, org_id)
    empty = False
    index = 0
    iterated = 0
    params = kwargs
    start_time = datetime.utcnow()
    
    while not empty:
        params['from'] = index
        params['limit'] = 100
        response = requests.get(url=url, headers=headers, params=params)
        if response.status_code == 401:
            token.generate()
            headers = make_header(token, org_id)
            continue
        content = json.loads(response.content.decode('utf-8'))
        data = content.get('data')
        if len(data) == 0:
            print("Done")
            empty = True

        for record in data:
            token, callback_response = callback(token, org_id, record)
            print(callback_response)
            index += 1
            print(f'{index} Records iterated')
        if len(data) < 100:
            empty = True

    end_time = datetime.utcnow()
    time_diff = end_time - start_time
    return token, f'{index} Records iterated.\nStart Time: {start_time.isoformat()}\nEnd Time: {end_time.isoformat()}\nElapsed: {time_diff}'

        
            
        
    



import requests
import json
import csv
from datetime import datetime
import os


def make_header(token, environment="development"):
    return {
        'Authorization': f'Zoho-oauthtoken {token.access}',
        'environment': environment
    }


class DataOperations:
    def __init__(self, token, environment, account_link_name, app_link_name, form_link_name):
        self.token = token
        self.environment = environment
        self.account_name = account_link_name
        self.app_name = app_link_name
        self.form_name = form_link_name
        self.result = None
        self.last_operation = None
        

    def add_record(self, data_object):
        url = f'https://creator.zoho.com/api/v2/{self.account_name}/{self.app_name}/form/{self.form_name}'
        headers = make_header(token, environment=self.environment)
        data = json.dumps(data_object).encode('utf-8')

        self.response = requests.post(url=url, headers=headers, data=data)
        if response.status_code == 401:
            self.token.generate()
            self.add_record(data_object)
        else:
            content = json.loads(response.content.decode('utf-8'))
            self.result = content['result'][0]


    def add_records(self, data_object_list):
        url = 
                                 
